@@ -4,7 +4,7 @@
 // Author:  Tad E. Smith
 //
 //
-// Copyright 2001-2015 Tad E. Smith
+// Copyright 2001-2017 Tad E. Smith
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -30,6 +30,10 @@
 #include <limits>
 #include <cstdlib>
 
+#if defined (LOG4CPLUS_WITH_UNIT_TESTS)
+#include <catch.hpp>
+#endif
+
 
 namespace
 {
@@ -40,12 +44,12 @@ log4cplus::tstring
 get_basename (const log4cplus::tstring& filename)
 {
 #if defined(_WIN32)
-    log4cplus::tchar const dir_sep(LOG4CPLUS_TEXT('\\'));
+    log4cplus::tstring const dir_sep(LOG4CPLUS_TEXT("\\/"));
 #else
     log4cplus::tchar const dir_sep(LOG4CPLUS_TEXT('/'));
 #endif
 
-    log4cplus::tstring::size_type pos = filename.rfind(dir_sep);
+    log4cplus::tstring::size_type pos = filename.find_last_of (dir_sep);
     if (pos != log4cplus::tstring::npos)
         return filename.substr(pos+1);
     else
@@ -111,7 +115,8 @@ private:
 };
 
 
-typedef std::vector<pattern::PatternConverter*> PatternConverterList;
+typedef std::vector<std::unique_ptr<pattern::PatternConverter> >
+PatternConverterList;
 
 
 /**
@@ -120,7 +125,8 @@ typedef std::vector<pattern::PatternConverter*> PatternConverterList;
 class LiteralPatternConverter : public PatternConverter
 {
 public:
-    LiteralPatternConverter(const tstring& str);
+    LiteralPatternConverter();
+    explicit LiteralPatternConverter(const tstring& str);
     virtual void convert(tstring & result,
         const spi::InternalLoggingEvent&)
     {
@@ -285,7 +291,7 @@ class PatternParser
 {
 public:
     PatternParser(const tstring& pattern, unsigned ndcMaxDepth);
-    std::vector<PatternConverter*> parse();
+    PatternConverterList parse();
 
 private:
   // Types
@@ -303,7 +309,7 @@ private:
   // Data
     tstring pattern;
     FormattingInfo formattingInfo;
-    std::vector<PatternConverter*> list;
+    PatternConverterList list;
     ParserState state;
     tstring::size_type pos;
     tstring currentLiteral;
@@ -387,6 +393,12 @@ PatternConverter::formatAndAppend(
 // LiteralPatternConverter methods:
 ////////////////////////////////////////////////
 
+LiteralPatternConverter::LiteralPatternConverter()
+    : PatternConverter(FormattingInfo())
+    , str()
+{ }
+
+
 LiteralPatternConverter::LiteralPatternConverter(
     const tstring& str_)
     : PatternConverter(FormattingInfo())
@@ -507,12 +519,12 @@ LoggerPatternConverter::convert(tstring & result,
         result = name;
     }
     else {
-        std::size_t len = name.length();
+        auto len = name.length();
 
         // We substract 1 from 'len' when assigning to 'end' to avoid out of
         // bounds exception in return r.substring(end+1, len). This can happen
         // if precision is 1 and the logger name ends with a dot.
-        tstring::size_type end = len - 1;
+        auto end = len - 1;
         for (int i = precision; i > 0; --i)
         {
             end = name.rfind(LOG4CPLUS_TEXT('.'), end - 1);
@@ -521,7 +533,7 @@ LoggerPatternConverter::convert(tstring & result,
                 return;
             }
         }
-        result = name.substr(end + 1);
+        result.assign (name, end + 1, tstring::npos);
     }
 }
 
@@ -547,7 +559,8 @@ void
 DatePatternConverter::convert(tstring & result,
     const spi::InternalLoggingEvent& event)
 {
-    result = event.getTimestamp().getFormattedTime(format, use_gmtime);
+    result = helpers::getFormattedTime(format, event.getTimestamp(),
+        use_gmtime);
 }
 
 
@@ -591,7 +604,7 @@ RelativeTimestampConverter::convert (tstring & result,
     tostringstream & oss = internal::get_ptd ()->layout_oss;
     detail::clear_tostringstream (oss);
     log4cplus::formatRelativeTimestamp (oss, event);
-    oss.str ().swap (result);
+    result = oss.str ();
 }
 
 
@@ -639,12 +652,10 @@ log4cplus::pattern::MDCPatternConverter::convert (tstring & result,
         result.clear ();
 
         MappedDiagnosticContextMap const & mdcMap = event.getMDCCopy();
-
-        for (MappedDiagnosticContextMap::const_iterator it = mdcMap.begin();
-             it != mdcMap.end(); ++it)
+        for (auto const & kv : mdcMap)
         {
-            tstring const & name(it->first);
-            tstring const & value(it->second);
+            tstring const & name = kv.first;
+            tstring const & value = kv.second;
 
             result += LOG4CPLUS_TEXT("{");
             result += name;
@@ -681,7 +692,7 @@ log4cplus::pattern::NDCPatternConverter::convert (tstring & result,
         for (int i = 1; i < precision && p != tstring::npos; ++i)
             p = text.find(LOG4CPLUS_TEXT(' '), p + 1);
 
-        result = text.substr(0, p);
+        result.assign (text, 0, p);
     }
 }
 
@@ -705,12 +716,14 @@ PatternParser::PatternParser(
 tstring
 PatternParser::extractOption()
 {
+    tstring r;
+
     if (   (pos < pattern.length())
         && (pattern[pos] == LOG4CPLUS_TEXT('{')))
     {
         tstring::size_type end = pattern.find_first_of(LOG4CPLUS_TEXT('}'), pos);
         if (end != tstring::npos) {
-            tstring r = pattern.substr(pos + 1, end - pos - 1);
+            r.assign (pattern, pos + 1, end - pos - 1);
             pos = end + 1;
             return r;
         }
@@ -724,7 +737,7 @@ PatternParser::extractOption()
         }
     }
 
-    return LOG4CPLUS_TEXT("");
+    return r;
 }
 
 
@@ -744,10 +757,9 @@ PatternParser::extractPrecisionOption()
 PatternConverterList
 PatternParser::parse()
 {
-    tchar c;
     pos = 0;
     while(pos < pattern.length()) {
-        c = pattern[pos++];
+        tchar const c = pattern[pos++];
         switch (state) {
         case LITERAL_STATE :
             // In literal state, the last char is always a literal.
@@ -765,7 +777,8 @@ PatternParser::parse()
                 default:
                     if(! currentLiteral.empty ()) {
                         list.push_back
-                             (new LiteralPatternConverter(currentLiteral));
+                            (std::unique_ptr<PatternConverter>(
+                                new LiteralPatternConverter(currentLiteral)));
                         //getLogLog().debug("Parsed LITERAL converter: \""
                         //                  +currentLiteral+"\".");
                     }
@@ -846,11 +859,13 @@ PatternParser::parse()
     } // end while
 
     if(! currentLiteral.empty ()) {
-        list.push_back(new LiteralPatternConverter(currentLiteral));
+        list.push_back(
+            std::unique_ptr<PatternConverter>(
+                new LiteralPatternConverter(currentLiteral)));
       //getLogLog().debug("Parsed LITERAL converter: \""+currentLiteral+"\".");
     }
 
-    return list;
+    return std::move (list);
 }
 
 
@@ -858,7 +873,7 @@ PatternParser::parse()
 void
 PatternParser::finalizeConverter(tchar c)
 {
-    PatternConverter* pc = 0;
+    PatternConverter* pc = nullptr;
     switch (c) {
         case LOG4CPLUS_TEXT('b'):
             pc = new BasicPatternConverter
@@ -1016,7 +1031,7 @@ PatternParser::finalizeConverter(tchar c)
             pc = new LiteralPatternConverter(currentLiteral);
     }
 
-    list.push_back(pc);
+    list.push_back(std::unique_ptr<PatternConverter>(pc));
     currentLiteral.resize(0);
     state = LITERAL_STATE;
     formattingInfo.reset();
@@ -1078,36 +1093,31 @@ PatternLayout::init(const tstring& pattern_, unsigned ndcMaxDepth)
     // Let's validate that our parser didn't give us any NULLs.  If it did,
     // we will convert them to a valid PatternConverter that does nothing so
     // at least we don't core.
-    for(PatternConverterList::iterator it=parsedPattern.begin();
-        it!=parsedPattern.end();
-        ++it)
+    for (auto & pc : parsedPattern)
     {
-        if( (*it) == 0 ) {
+        if (! pc)
+        {
             helpers::getLogLog().error(
                 LOG4CPLUS_TEXT("Parsed Pattern created a NULL PatternConverter"));
-            (*it) = new pattern::LiteralPatternConverter( LOG4CPLUS_TEXT("") );
+            pc.reset (new pattern::LiteralPatternConverter);
         }
     }
-    if(parsedPattern.empty ()) {
+
+    if(parsedPattern.empty ())
+    {
         helpers::getLogLog().warn(
             LOG4CPLUS_TEXT("PatternLayout pattern is empty.  Using default..."));
         parsedPattern.push_back (
-            new pattern::BasicPatternConverter(pattern::FormattingInfo(),
-            pattern::BasicPatternConverter::MESSAGE_CONVERTER));
+            std::unique_ptr<pattern::PatternConverter>(
+                new pattern::BasicPatternConverter(pattern::FormattingInfo(),
+                    pattern::BasicPatternConverter::MESSAGE_CONVERTER)));
     }
 }
 
 
 
 PatternLayout::~PatternLayout()
-{
-    for(PatternConverterList::iterator it=parsedPattern.begin();
-        it!=parsedPattern.end();
-        ++it)
-    {
-        delete (*it);
-    }
-}
+{ }
 
 
 
@@ -1115,13 +1125,25 @@ void
 PatternLayout::formatAndAppend(tostream& output,
                                const spi::InternalLoggingEvent& event)
 {
-    for(PatternConverterList::iterator it=parsedPattern.begin();
-        it!=parsedPattern.end();
-        ++it)
+    for (auto const & pc : parsedPattern)
     {
-        (*it)->formatAndAppend(output, event);
+        pc->formatAndAppend(output, event);
     }
 }
 
+#if defined (LOG4CPLUS_WITH_UNIT_TESTS)
+CATCH_TEST_CASE ("PatternLayout", "[patternlayout]")
+{
+    CATCH_SECTION ("get_basename")
+    {
+        CATCH_REQUIRE(::get_basename(LOG4CPLUS_TEXT("/a/b")) == LOG4CPLUS_TEXT("b"));
+        #ifdef _WIN32
+        CATCH_REQUIRE(::get_basename(LOG4CPLUS_TEXT("C:/a/b")) == LOG4CPLUS_TEXT("b"));
+        CATCH_REQUIRE(::get_basename(LOG4CPLUS_TEXT("C:\\a\\b")) == LOG4CPLUS_TEXT("b"));
+        #endif
+    }
+}
+
+#endif
 
 } // namespace log4cplus
